"""Scripts to set the sample date udfs.
"""
import logging
import sys
from typing import List, Optional

import click

import requests
from genologics.entities import Process

from cg_lims import options
from cg_lims.exceptions import LimsError, MissingUDFsError, AtlasResponseFailedError

LOG = logging.getLogger(__name__)


def get_path(document_udf: str, process: Process, atlas_host: str) -> Optional[str]:
    """Get method document path from Atlas"""

    document_title: str = process.udf.get(document_udf)
    if not document_title:
        LOG.warning(f"Process Udf: {document_udf} does not exist or was left empty.")
        return
    response = requests.get(f"{atlas_host}/title/{document_title}/path")
    if response.status_code != 200:
        raise AtlasResponseFailedError(message=f"{response.status_code} : {response.text}")
    return response.json()


def get_document_paths(document_udfs: List[str], process: Process, host: str) -> List[str]:
    """Get method document paths from Atlas"""

    method_documents: List[Optional[str]] = [get_path(udf, process, host) for udf in document_udfs]
    return [doc for doc in method_documents if doc]


def set_methods_and_version(document_udfs: List[str], process: Process, host: str) -> None:
    """Set Method Documents and Atlas Version"""

    method_documents = get_document_paths(document_udfs=document_udfs, host=host, process=process)
    if not method_documents:
        raise MissingUDFsError(message="Found no valid Method document UDFs in the step.")
    response = requests.get(f"{host}/version")
    if response.status_code != 200:
        raise AtlasResponseFailedError(message=f"{response.status_code} : {response.text}")
    process.udf["Methods"] = " ,".join(method_documents)
    process.udf["Atlas Version"] = response.json()
    process.put()
    LOG.info("Method Documents and Atlas Version have been set.")
    click.echo("Method Documents and Atlas Version have been set.")


@click.command()
@options.process_udfs(help="List of Method Document UDFs.")
@click.pass_context
def method_document(ctx: click.Context, process_udfs: List[str]):
    """Script to set method documents and atlas version."""

    LOG.info(f"Running {ctx.command_path} with params: {ctx.params}")
    process: Process = ctx.obj["process"]
    atlas_host: str = ctx.obj["atlas_host"]
    try:
        set_methods_and_version(process=process, host=atlas_host, document_udfs=process_udfs)
    except LimsError as e:
        sys.exit(e.message)



from genologics.lims import Lims
from pydantic.main import BaseModel
from pydantic import Field
from cg_lims.models.arnold.prep.base_step import BaseStep
from cg_lims.objects import BaseAnalyte


class LibraryPrepNexteraProcessUDFS(BaseModel):
    lot_nr_tagmentation_buffer: str = Field(..., alias="Lot nr: Tagmentation buffer (TD-buffer)")
    lot_nr_tagmentation_enzyme: str = Field(..., alias="Lot nr: Tagmentation enzyme (TDE1)")
    lot_nr_index: str = Field(..., alias="Lot nr: Index")
    lot_nr_pcr_mix: str = Field(..., alias="Lot nr: KAPA HiFi HotStart ReadyMix (2X)")
    pcr_instrument_incubation: str = Field(..., alias="PCR instrument incubation")
    pcr_instrument_amplification: str = Field(..., alias="PCR instrument amplification")
    nr_pcr_cycles: int = Field(..., alias="Nr PCR cycles")


class LibraryPrepFields(BaseStep):
    process_udfs: LibraryPrepNexteraProcessUDFS

    class Config:
        allow_population_by_field_name = True


def get_library_prep_nextera(lims: Lims, sample_id: str, prep_id: str) -> LibraryPrepFields:
    analyte = BaseAnalyte(
        lims=lims,
        sample_id=sample_id,
        process_type="CG002 - Microbial Library Prep (Nextera)",
    )

    return LibraryPrepFields(
        **analyte.base_fields(),
        process_udfs=LibraryPrepNexteraProcessUDFS(**analyte.process_udfs()),
        sample_id=sample_id,
        prep_id=prep_id,
        step_type="library_prep",
        workflow="Microbial"
    )



from typing import Optional

from genologics.lims import Lims
from pydantic import Field, BaseModel

from cg_lims.objects import BaseAnalyte

from cg_lims.models.arnold.prep.base_step import BaseStep


class BufferExchangeArtifactUDFs(BaseModel):
    concentration: Optional[float] = Field(None, alias="Concentration")


class BufferExchangeFields(BaseStep):
    artifact_udfs: BufferExchangeArtifactUDFs

    class Config:
        allow_population_by_field_name = True


def get_buffer_exchange_twist(
    lims: Lims, sample_id: str, prep_id: str
) -> Optional[BufferExchangeFields]:
    analyte = BaseAnalyte(
        lims=lims,
        sample_id=sample_id,
        process_type="Buffer Exchange v2",
        optional_step=True,
    )

    if not analyte.artifact:
        return None

    return BufferExchangeFields(
        **analyte.base_fields(),
        artifact_udfs=BufferExchangeArtifactUDFs(**analyte.artifact_udfs()),
        sample_id=sample_id,
        prep_id=prep_id,
        step_type="buffer_exchange",
        workflow="TWIST",
    )



from typing import List

from genologics.lims import Lims

from .buffer_exchange import (
    BufferExchangeFields,
    BufferExchangeArtifactUDFs,
    get_buffer_exchange_twist,
)
from .capture_and_wash_twist import (
    CaptureandWashFields,
    CaptureAndWashProcessUDFs,
    get_capture_and_wash,
)
from .pool_samples_twist import (
    PoolSamplesForHybridizationFields,
    PoolsamplesforhybridizationArtifactUDFs,
    get_pool_samples_twist,
)
from .hybridize_library_twist import (
    HybridizeLibraryTWISTProcessUDFs,
    HybridizeLibraryFields,
    HybridizeLibraryTWISTArtifactUDFs,
    get_hybridize_library_twist,
)
from .bead_purification_twist import (
    BeadPurificationFields,
    BeadPurificationArtifactUDFs,
    get_bead_purification_twist,
)
from .kapa_library_preparation_twist import (
    KAPALibraryPreparationProcessUDFs,
    KAPALibraryPreparationArtifactUDFs,
    KAPALibraryPreparationFields,
    get_kapa_library_preparation_twist,
)
from .aliquot_samples_for_enzymatic_fragmentation_twist import (
    AliquotSamplesForEnzymaticFragmentationFields,
    AliquotSamplesForEnzymaticFragmentationArtifactUdfs,
    AliquotSamplesForEnzymaticFragmentationProcessUdfs,
    get_aliquot_samples_for_enzymatic_fragmentation,
)

from .amplify_captured_libraries import (
    AmplifycapturedLibrariestwistFields,
    AmplifycapturedLibrariestwistProcessUDFs,
    get_amplify_captured_library_udfs,
)

from .enzymatic_fragmentation_twist import (
    EnzymaticFragmentationTWISTFields,
    EnzymaticFragmentationTWISTProcessUdfs,
    get_enzymatic_fragmentation,
)

from .reception_control import (
    SampleArtifactUDF,
    SampleArtifactFields,
    get_sample_artifact_fields,
)
from ..base_step import BaseStep


def build_twist_documents(sample_id: str, process_id: str, lims: Lims) -> List[BaseStep]:
    """Building a sars_cov_2 Prep."""
    prep_id = f"{sample_id}_{process_id}"
    step_documents = [
        get_aliquot_samples_for_enzymatic_fragmentation(
            sample_id=sample_id, lims=lims, prep_id=prep_id
        ),
        get_hybridize_library_twist(sample_id=sample_id, lims=lims, prep_id=prep_id),
        get_pool_samples_twist(sample_id=sample_id, lims=lims, prep_id=prep_id),
        get_capture_and_wash(sample_id=sample_id, lims=lims, prep_id=prep_id),
        get_kapa_library_preparation_twist(sample_id=sample_id, lims=lims, prep_id=prep_id),
        get_buffer_exchange_twist(sample_id=sample_id, lims=lims, prep_id=prep_id),
        get_bead_purification_twist(sample_id=sample_id, lims=lims, prep_id=prep_id),
        get_enzymatic_fragmentation(sample_id=sample_id, lims=lims, prep_id=prep_id),
        get_amplify_captured_library_udfs(sample_id=sample_id, lims=lims, prep_id=prep_id),
        get_sample_artifact_fields(sample_id=sample_id, lims=lims, prep_id=prep_id),
    ]
    return [document for document in step_documents if document is not None]
